#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Automaton.h"
#include "CutDouble.h"
#ifdef PROTOS_NEEDED
#include "Suggest.h"
#endif



DAGElement *Automaton=NULL ;
ULONG AutomatonSize ;


extern UBYTE MaxDiffLenPlus ;
extern UBYTE MaxDiffLenLess ;
extern UBYTE MaxDefaultsAllowed ;
extern UBYTE MaxConsecutiveDefaults ;
extern UBYTE MaxDefaultAllowedTable[] ;
extern UBYTE MaxConsecutiveDefaultsTable[] ;
extern UBYTE MaxDiffLenPlusTable[] ;
extern UBYTE MaxDiffLenLessTable[] ;
extern UBYTE Buffer[MAX_WORD_LEN] ;
extern UBYTE Word[MAX_WORD_LEN] ;
extern BYTE WordLen ;
extern struct StringItem *TheWordList ;
extern char *yytext ;
extern unsigned long CurrentLineNumber ;


void main (argc, argv)
	int argc ;
	char *argv[] ;

{
FILE *InHd ;
ULONG InitialState ;
ULONG Magic ;
UBYTE i ;

fprintf (stderr, "Merime - Spelling checker/corrector by Franois PESSAUX (c) 1995.\n") ;
if (argc != 2)
  {
    printf ("Parameter error. Usage: Merime <DicFile>.\n") ;
    printf ("DicFile must be a dictionary data base.\n") ;
    printf ("Merime take a dictionary database as argument, your text ") ;
    printf ("on the standard input, and gives mis-spelling informations") ;
    printf (" and suggestions on standard output\n") ;
    printf ("Typical use : Merime myDic.DAG < mytext.txt | more\n") ;
    exit (0) ;
  }

InHd = (FILE*) fopen (argv[1], "r") ;
if (InHd == NULL)
  {
    printf ("Reading error.\n") ;
    printf ("Can't open the dictionary file '%s'for reading.\n", argv[1]) ;
    exit (0) ;
  }

fread (&Magic, 1, sizeof (ULONG), InHd) ;
if (Magic != TABLEMAGICNUMBER)
  {
    printf ("Format error.\n") ;
    printf ("File '%s' isn't a dictionary file.\n", argv[1]) ;
    fclose (InHd) ;
    exit (0) ;
  }

fread (&AutomatonSize, 1, sizeof (ULONG), InHd) ;
fread (&InitialState, 1, sizeof (ULONG), InHd) ;
Automaton = (DAGElement*) malloc (AutomatonSize*sizeof (DAGElement)) ;
fread (Automaton, AutomatonSize, sizeof (DAGElement), InHd) ;
fclose (InHd) ;



memset (Word, 0, MAX_WORD_LEN) ;
while (yylex ()!=0)
  {
    strcpy (Word,yytext) ;
    for (i=0;i<strlen(Word);i++)
      { Word[i] = (unsigned char) tolower (Word[i]) ;}
      
    if (!Exists (InitialState, Word))
      {
	printf ("Line : %ld, Unknown : '%s'\n", CurrentLineNumber, Word) ;
	
	/* DECLARATIONS GLOBALES POUR EVITER L'ENGORGEMENT DE LA PILE */
	/* ET POUR AUGMENTER LA VITESSE*/
	WordLen = (BYTE) strlen (Word) ;
	
	MaxDiffLenPlus = MaxDiffLenPlusTable[WordLen] ;
	MaxDiffLenLess = MaxDiffLenLessTable[WordLen] ;
	MaxDefaultsAllowed = MaxDefaultAllowedTable[WordLen] ;
	MaxConsecutiveDefaults = MaxConsecutiveDefaultsTable[WordLen] ;
	
	Suggest (InitialState, 0, 0, 0, 0, FIRST_RUN, '\0', FALSE) ;
	printf ("\n\n") ;
	ResetWordList (&TheWordList) ;
      }
    memset (Word, 0, MAX_WORD_LEN) ;
  }

/* ON LIBERE LA MEMOIRE L'HISTOIRE D'ETRE SYMPA */
free (Automaton) ;
printf ("End of processing.\n") ;
exit (0) ;
}

